"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var redux_actions_1 = require("redux-actions");
var imm = require("object-path-immutable");
var Observable_1 = require("rxjs/Observable");
var constants_1 = require("./constants");
var state_mutation_1 = require("./state-mutation");
var utils_1 = require("./utils");
var API_SET_ENDPOINT_HOST = constants_1.default.API_SET_ENDPOINT_HOST, API_SET_ENDPOINT_PATH = constants_1.default.API_SET_ENDPOINT_PATH, API_SET_HEADERS = constants_1.default.API_SET_HEADERS, API_SET_HEADER = constants_1.default.API_SET_HEADER, API_WILL_CREATE = constants_1.default.API_WILL_CREATE, API_CREATED = constants_1.default.API_CREATED, API_CREATE_FAILED = constants_1.default.API_CREATE_FAILED, API_WILL_READ = constants_1.default.API_WILL_READ, API_READ = constants_1.default.API_READ, API_READ_FAILED = constants_1.default.API_READ_FAILED, API_WILL_UPDATE = constants_1.default.API_WILL_UPDATE, API_UPDATED = constants_1.default.API_UPDATED, API_UPDATE_FAILED = constants_1.default.API_UPDATE_FAILED, API_WILL_DELETE = constants_1.default.API_WILL_DELETE, API_DELETED = constants_1.default.API_DELETED, API_DELETE_FAILED = constants_1.default.API_DELETE_FAILED;
exports.IS_DELETING = "IS_DELETING";
exports.IS_UPDATING = "IS_UPDATING";
exports.setEndpointHost = redux_actions_1.createAction(API_SET_ENDPOINT_HOST);
exports.setEndpointPath = redux_actions_1.createAction(API_SET_ENDPOINT_PATH);
exports.setHeaders = redux_actions_1.createAction(API_SET_HEADERS);
exports.setHeader = redux_actions_1.createAction(API_SET_HEADER);
var apiWillCreate = redux_actions_1.createAction(API_WILL_CREATE);
var apiCreated = redux_actions_1.createAction(API_CREATED);
var apiCreateFailed = redux_actions_1.createAction(API_CREATE_FAILED);
var apiWillRead = redux_actions_1.createAction(API_WILL_READ);
var apiRead = redux_actions_1.createAction(API_READ);
var apiReadFailed = redux_actions_1.createAction(API_READ_FAILED);
var apiWillUpdate = redux_actions_1.createAction(API_WILL_UPDATE);
var apiUpdated = redux_actions_1.createAction(API_UPDATED);
var apiUpdateFailed = redux_actions_1.createAction(API_UPDATE_FAILED);
var apiWillDelete = redux_actions_1.createAction(API_WILL_DELETE);
var apiDeleted = redux_actions_1.createAction(API_DELETED);
var apiDeleteFailed = redux_actions_1.createAction(API_DELETE_FAILED);
exports.setAccessToken = function (at) {
    return function (dispatch) {
        dispatch(exports.setHeader({ Authorization: "Bearer " + at }));
    };
};
exports.createResource = function (resource) {
    return function (dispatch, getState) {
        dispatch(apiWillCreate(resource));
        var _a = getState().api.endpoint, apiHost = _a.host, apiPath = _a.path, headers = _a.headers;
        var endpoint = "" + apiHost + apiPath + "/" + resource.type;
        return utils_1.apiRequest(endpoint, {
            headers: headers,
            method: "POST",
            withCredentials: true,
            body: JSON.stringify({
                data: resource
            })
        })
            .switchMap(function (json) {
            dispatch(apiCreated(json));
            return Observable_1.Observable.of(json);
        })
            .catch(function (error) {
            var err = error;
            err.resource = resource;
            dispatch(apiCreateFailed(err));
            return Observable_1.Observable.throw(err);
        });
    };
};
exports.readEndpoint = function (endpoint, _a) {
    var _b = (_a === void 0 ? {} : _a).options, options = _b === void 0 ? {
        indexLinks: undefined
    } : _b;
    return function (dispatch, getState) {
        dispatch(apiWillRead(endpoint));
        var _a = getState().api.endpoint, apiHost = _a.host, apiPath = _a.path, headers = _a.headers;
        var apiEndpoint = "" + apiHost + apiPath + "/" + endpoint;
        return utils_1.apiRequest("" + apiEndpoint, {
            headers: headers,
            withCredentials: true
        })
            .switchMap(function (json) {
            dispatch(apiRead(tslib_1.__assign({ endpoint: endpoint, options: options }, json)));
            return Observable_1.Observable.of(json);
        })
            .catch(function (error) {
            var err = error;
            err.endpoint = endpoint;
            dispatch(apiReadFailed(err));
            return Observable_1.Observable.throw(err);
        });
    };
};
exports.updateResource = function (resource) {
    return function (dispatch, getState) {
        dispatch(apiWillUpdate(resource));
        var _a = getState().api.endpoint, apiHost = _a.host, apiPath = _a.path, headers = _a.headers;
        var endpoint = "" + apiHost + apiPath + "/" + resource.type + "/" + resource.id;
        return utils_1.apiRequest(endpoint, {
            headers: headers,
            method: "PATCH",
            withCredentials: true,
            body: JSON.stringify({
                data: resource
            })
        })
            .switchMap(function (json) {
            dispatch(apiUpdated(json));
            return Observable_1.Observable.of(json);
        })
            .catch(function (error) {
            var err = error;
            err.resource = resource;
            dispatch(apiUpdateFailed(err));
            return Observable_1.Observable.throw(err);
        });
    };
};
exports.deleteResource = function (resource) {
    return function (dispatch, getState) {
        dispatch(apiWillDelete(resource));
        var _a = getState().api.endpoint, apiHost = _a.host, apiPath = _a.path, headers = _a.headers;
        var endpoint = "" + apiHost + apiPath + "/" + resource.type + "/" + resource.id;
        return utils_1.apiRequest(endpoint, {
            headers: headers,
            method: "DELETE",
            withCredentials: true
        })
            .switchMap(function () {
            dispatch(apiDeleted(resource));
            return Observable_1.Observable.of();
        })
            .catch(function (error) {
            var err = error;
            err.resource = resource;
            dispatch(apiDeleteFailed(err));
            return Observable_1.Observable.throw(err);
        });
    };
};
exports.requireResource = function (resourceType, endpoint) {
    if (endpoint === void 0) { endpoint = resourceType; }
    return function (dispatch, getState) {
        var api = getState().api;
        if (api.hasOwnProperty(resourceType)) {
            return Observable_1.Observable.of();
        }
        return dispatch(exports.readEndpoint(endpoint));
    };
};
exports.reducer = redux_actions_1.handleActions((_a = {},
    _a[API_SET_HEADERS] = function (state, _a) {
        var headers = _a.payload;
        return imm(state)
            .set(["endpoint", "headers"], headers)
            .value();
    },
    _a[API_SET_HEADER] = function (state, _a) {
        var header = _a.payload;
        var newState = imm(state);
        Object.keys(header).forEach(function (key) {
            newState.set(["endpoint", "headers", key], header[key]);
        });
        return newState.value();
    },
    _a[API_SET_ENDPOINT_HOST] = function (state, _a) {
        var host = _a.payload;
        return imm(state)
            .set(["endpoint", "host"], host)
            .value();
    },
    _a[API_SET_ENDPOINT_PATH] = function (state, _a) {
        var path = _a.payload;
        return imm(state)
            .set(["endpoint", "path"], path)
            .value();
    },
    _a[API_WILL_CREATE] = function (state) {
        return imm(state)
            .set(["isCreating"], state.isCreating + 1)
            .value();
    },
    _a[API_CREATED] = function (state, _a) {
        var resources = _a.payload;
        var entities = Array.isArray(resources.data)
            ? resources.data
            : [resources.data];
        var newState = state_mutation_1.updateOrInsertResourcesIntoState(state, entities.concat(resources.included || []));
        return imm(newState)
            .set("isCreating", state.isCreating - 1)
            .value();
    },
    _a[API_CREATE_FAILED] = function (state) {
        return imm(state)
            .set(["isCreating"], state.isCreating - 1)
            .value();
    },
    _a[API_WILL_READ] = function (state) {
        return imm(state)
            .set(["isReading"], state.isReading + 1)
            .value();
    },
    _a[API_READ] = function (state, _a) {
        var payload = _a.payload;
        var resources = (Array.isArray(payload.data)
            ? payload.data
            : [payload.data]).concat(payload.included || []);
        var newState = state_mutation_1.updateOrInsertResourcesIntoState(state, resources);
        var finalState = state_mutation_1.addLinksToState(newState, payload.links, payload.options);
        return imm(finalState)
            .set("isReading", state.isReading - 1)
            .value();
    },
    _a[API_READ_FAILED] = function (state) {
        return imm(state)
            .set(["isReading"], state.isReading - 1)
            .value();
    },
    _a[API_WILL_UPDATE] = function (state, _a) {
        var resource = _a.payload;
        var type = resource.type, id = resource.id;
        var newState = state_mutation_1.ensureResourceTypeInState(state, type);
        return state_mutation_1.setIsInvalidatingForExistingResource(newState, { type: type, id: id }, exports.IS_UPDATING)
            .set("isUpdating", state.isUpdating + 1)
            .value();
    },
    _a[API_UPDATED] = function (state, _a) {
        var resources = _a.payload;
        var entities = Array.isArray(resources.data)
            ? resources.data
            : [resources.data];
        var newState = state_mutation_1.updateOrInsertResourcesIntoState(state, entities.concat(resources.included || []));
        return imm(newState)
            .set("isUpdating", state.isUpdating - 1)
            .value();
    },
    _a[API_UPDATE_FAILED] = function (state, _a) {
        var resource = _a.payload.resource;
        var type = resource.type, id = resource.id;
        return state_mutation_1.setIsInvalidatingForExistingResource(state, { type: type, id: id }, exports.IS_UPDATING)
            .set("isUpdating", state.isUpdating + 1)
            .value();
    },
    _a[API_WILL_DELETE] = function (state, _a) {
        var resource = _a.payload;
        var type = resource.type, id = resource.id;
        return state_mutation_1.setIsInvalidatingForExistingResource(state, { type: type, id: id }, exports.IS_DELETING)
            .set("isDeleting", state.isDeleting + 1)
            .value();
    },
    _a[API_DELETED] = function (state, _a) {
        var resource = _a.payload;
        return state_mutation_1.removeResourceFromState(state, resource)
            .set("isDeleting", state.isDeleting - 1)
            .value();
    },
    _a[API_DELETE_FAILED] = function (state, _a) {
        var resource = _a.payload.resource;
        var type = resource.type, id = resource.id;
        return state_mutation_1.setIsInvalidatingForExistingResource(state, { type: type, id: id }, exports.IS_DELETING)
            .set("isDeleting", state.isDeleting + 1)
            .value();
    },
    _a), {
    isCreating: 0,
    isReading: 0,
    isUpdating: 0,
    isDeleting: 0,
    endpoint: {
        host: null,
        path: null,
        headers: {
            "Content-Type": "application/vnd.api+json",
            Accept: "application/vnd.api+json"
        }
    }
});
var _a;
//# sourceMappingURL=jsonapi.js.map